""" Citrix Netscaler Nitro API accessor """

import urllib, urllib2
from json import JSONEncoder
from nsutil import *
from nsresources import *

__version__ = "0.0.2"

class NSNitro:
        """ Main class """

        __ip      = "1.2.3.4"
        __user  = "api_user"
        __password    = "api_user"
        __baseurl     = "http://1.2.3.4/nitro/v1/config/"
        __sessionid   = ""
        __loggedin    = False
        __initialized = False
        __contenttype = "application/x-www-form-urlencoded"
        __postheaders = {'Cookie' : 'sessionid='+__sessionid, 'Content-type' : __contenttype}

        def __init__(self, ip, user, password, useSSL=False):
                """ Contructor: ip - LB ip, user - LB username, pass - LB password """
                self.__ip = ip
                self.__user = user
                self.__password = password
                self.__baseurl = "%s://%s/nitro/v1/config/" % ('https' if useSSL else 'http',ip)
                self.__initialized = True

        def get_url(self):
                """ Returns base url for nitro API. Mostly useful for debugging """
                if not self.__initialized:
                        raise NSNitroError("Not initialized.")
                return self.__baseurl

        def get_sessionid(self):
                """ Returns sessionID that LB gave us after logging in """
                if not self.__initialized or not self.__loggedin:
                        raise NSNitroError("Not initialized or not logged in.")

                return self.__sessionid

        def login(self):
                """ Logins to the LB using the credentials give to constructor """
                if not self.__initialized:
                        raise NSNitroError("Not initialized.")

                payload = {"object":{"login":{"username":self.__user,"password":self.__password}}}
                try:
                        nsresponse = self.post(payload)
                        if nsresponse.failed:
                                raise NSNitroError(nsresponse.message)

                        self.__sessionid = nsresponse.get_response_field('sessionid')
                        self.__postheaders = {'Cookie' : 'sessionid='+self.__sessionid, 'Content-type' : self.__contenttype}
                        self.__loggedin = True
                        return True

                except SyntaxError:
                        raise NSNitroError("Could not parse LB response.")
                except urllib2.URLError, ue:
                        raise NSNitroError("Error logging in!" + ue.message)

        def post(self, payload):
                try:
                        payload_encoded = urllib.urlencode(payload)
                        req = urllib2.Request(self.__baseurl, payload_encoded, self.__postheaders)
                        response = urllib2.urlopen(req)

                except urllib2.HTTPError, e:
                        raise NSNitroError("Could not send post request: %s, %s" % (e.code, e.message))

                nsresponse = NSNitroResponse(response.read())
                if nsresponse.failed:
                        raise NSNitroError(nsresponse.message)
                return nsresponse

        def put(self, payload):
                try:
                        opener = urllib2.build_opener(urllib2.HTTPHandler)
                        request = urllib2.Request(self.__baseurl, json.dumps(payload))
                        request.add_header('Cookie', 'sessionid='+self.__sessionid)
                        request.get_method = lambda: 'PUT'
                        response = opener.open(request)

                except urllib2.HTTPError, e:
                        raise NSNitroError("Could not send put request: %s, %s" % (e.code, e.message))

                nsresponse = NSNitroResponse(response.read())
                if nsresponse.failed:
                        raise NSNitroError(nsresponse.message)
                return nsresponse

        def get(self, url):
                try:
                        opener = urllib2.build_opener()
                        opener.addheaders.append(('Cookie', 'sessionid='+self.__sessionid))
                        response = opener.open(url)

                except urllib2.HTTPError, e:
                        print "Got reponse code: %s from the server" % e.code
                        raise NSNitroError("Could not get resource: %s, %s" % (e.code, e.message))

                nsresponse = NSNitroResponse(response.read())
                if nsresponse.failed:
                        raise NSNitroError(nsresponse.message)

                return nsresponse

        def delete(self, url):
                try:
                        opener = urllib2.build_opener()
                        req = urllib2.Request(url)
                        req.add_header('Cookie', 'sessionid='+self.__sessionid)
                        req.get_method = lambda: 'DELETE'
                        response = urllib2.urlopen(req)

                except urllib2.HTTPError, e:
                        raise NSNitroError("Could not send delete request: %s, %s" % (e.code, e.message))

                nsresponse = NSNitroResponse(response.read())
                if nsresponse.failed:
                        raise NSNitroError(nsresponse.message)
                return nsresponse

        def logout(self):
                try:
                    opener = urllib2.build_opener()
                    req = urllib2.Request(self.__baseurl)
                    req.add_header('Cookie','sessionid='+self.__sessionid)
                    req.add_header('logout','{}')
                    response = urllib2.urlopen(req)
                except urllib2.HTTPError,e:
                    raise NSNitroError("Could not send logout request: %s, %s" % (e.code, e.message))

                nsresponse = NSNitroResponse(response.read())
                if nsresponse.failed:
                        raise NSNitroError(nsresponse.message)
                del self.__sessionid

                return nsresponse.get_json_response()



from nsbaseresource import NSBaseResource

__author__ = 'timl'

class NSResponderPolicy(NSBaseResource):

        # Configuration for content-switching policy resource.

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSResponderPolicy, self).__init__()
                self.options = {
                        'name' : '',
                        'rule' : '',
                        'action' : '',
                        'hits' : '',
                        'undefhits' : '',
                        'newname' : '',
                        'activepolicy' : '',
                        'priority' : '',
                }

                self.resourcetype = NSResponderPolicy.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "responderpolicy"

        def set_name(self, name):
                """
                Name of the responder policy.

                Default value: 0
                """
                self.options['name'] = name

        def get_name(self):
                """
                Name of the responder policy.

                Default value: 0
                """
                return self.options['name']

        def set_rule(self, rule):
                """
                Expression to be used by responder policy. It has to be a boolean PI rule expression.

                Default value: 0
                """
                self.options['rule'] = rule

        def get_rule(self):
                """
                Expression to be used by responder policy. It has to be a boolean PI rule expression.

                Default value: 0
                """
                return self.options['rule']

        def set_action(self, action):
                """
                Responder action to be used by the policy.

                Default value: 0
                """
                self.options['action'] = action

        def get_action(self):
                """
                Responder action to be used by the policy.

                Default value: 0
                """
                return self.options['action']

        def set_newname(self, newname):
                """
                The new name of the responder policy.

                Default value: 0
                Minimum length =  1.
                """
                self.options['newname'] = newname

        def get_newname(self):
                """
                The new name of the responder policy.

                Default value: 0
                Minimum length =  1.
                """
                return self.options['newname']

        def get_hits(self):
                """
                Number of hits.

                Default value: 0
                """
                return self.options['hits']

        def get_undefhits(self):
                """
                Number of undef hits.

                Default value: 0
                """
                return self.options['undefhits']

        def get_activepolicy(self):
                """
                Policy is in use.

                Default value: 0
                """
                return self.options['activepolicy']

        def set_priority(self, priority):
                """
                Set the priority for the responder policy.

                Default value: 0
                """
                self.options['priority'] = priority

        def get_priority(self):
                """
                Priority of the responder policy.

                Default value: 0
                """
                return self.options['priority']

        # Operations methods
        @staticmethod
        def get(nitro, responderpolicy):
                """
                Use this API to fetch responderpolicy resource of given name.
                """
                __responderpolicy = NSResponderPolicy()
                __responderpolicy.set_name(responderpolicy.get_name())
                __responderpolicy.get_resource(nitro)
                return __responderpolicy

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured responderpolicy resources.
                """
                __url = nitro.get_url() + NSResponderPolicy.get_resourcetype()
                __json_cspolicies = nitro.get(__url).get_response_field(NSResponderPolicy.get_resourcetype())
                __responderpolicies = []
                for json_responderpolicy in __json_cspolicies:
                        __responderpolicies.append(NSResponderPolicy(json_responderpolicy))
                return __responderpolicies

        @staticmethod
        def add(nitro, responderpolicy):
                """
                Use this API to add responderpolicy.
                """
                __responderpolicy = NSResponderPolicy()
                __responderpolicy.set_name(responderpolicy.get_name())
                __responderpolicy.set_rule(responderpolicy.get_rule())
                __responderpolicy.set_action(responderpolicy.get_action())
                __responderpolicy.set_priority(responderpolicy.get_priority())
                return __responderpolicy.add_resource(nitro)

        @staticmethod
        def delete(nitro, responderpolicy):
                """
                Use this API to delete responderpolicy of a given name.
                """
                __responderpolicy = NSResponderPolicy()
                __responderpolicy.set_name(responderpolicy.get_name())
                nsresponse = __responderpolicy.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, responderpolicy):
                """
                Use this API to update a responderpolicy of a given name.
                """
                __responderpolicy = NSResponderPolicy()
                __responderpolicy.set_name(responderpolicy.get_name())
                __responderpolicy.set_rule(responderpolicy.get_rule())
                __responderpolicy.set_action(responderpolicy.get_action())
                __responderpolicy.set_priority(responderpolicy.get_priority())
                return __responderpolicy.update_resource(nitro)


        # No unset functionality for now.
  
        @staticmethod
        def rename(nitro, responderpolicy):
                """
                Use this API to rename responderpolicy.
                """
                __responderpolicy = NSResponderPolicy()
                __responderpolicy.set_name(responderpolicy.get_name())
                __responderpolicy.set_newname(responderpolicy.get_newname())
                return __responderpolicy.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'

class NSCSPolicy(NSBaseResource):

        # Configuration for content-switching policy resource.

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSCSPolicy, self).__init__()
                options = {
                        'policyname' : '',
                        'url' : '',
                        'rule' : '',
                        'domain' : '',
                        'vstype' : '',
                        'hits' : '',
                        'bindhits' : '',
                        'labelname' : '',
                        'labeltype' : '',
                        'target' : '',
                        'priority' : '',
                        '__count' : '',
                }

                self.resourcetype = NSCSPolicy.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "cspolicy"

        def set_policyname(self, policyname):
                """
                The name of the new content switching policy.

                Default value: 0
                Minimum length =  1.
                """
                self.options['policyname'] = policyname

        def get_policyname(self):
                """
                The name of the new content switching policy.

                Default value: 0
                Minimum length =  1.
                """
                return self.options['policyname']

        def set_url(self, url):
                """
                The URL, with wildcards.
                Specify the string value in this format:
                // [[prefix ] [*]] [.suffix].

                Default value: 0
                Minimum length =  1.
                Maximum length =  208.
                """
                self.options['url'] = url

        def get_url(self):
                """
                The URL, with wildcards.
                Specify the string value in this format:
                // [[prefix ] [*]] [.suffix].

                Default value: 0
                Minimum length =  1.
                Maximum length =  208.
                """
                return self.options['url']

        def set_rule(self, rule):
                """
                The condition for applying this policy.
                Expression logic is as follows:
                        - Expression names separated by the logical operators || and &&.
                        - Expression names may be grouped using parenthesis.
                        - If the expression contains blanks (e.g., between an expression name and a logical operator),
                                then the entire argument must be enclosed in double quotes.
                The following example shows valid expression logic:
                        ns_ext_cgi||ns_ext_asp
                        "ns_non_get && (ns_header_cookie||ns_header_pragma)".

                Default value: 0
                """
                self.options['rule'] = rule

        def get_rule(self):
                """
                The condition for applying this policy.
                Expression logic is as follows:
                        - Expression names separated by the logical operators || and &&.
                        - Expression names may be grouped using parenthesis.
                        - If the expression contains blanks (e.g., between an expression name and a logical operator),
                                then the entire argument must be enclosed in double quotes.
                The following example shows valid expression logic:
                        ns_ext_cgi||ns_ext_asp
                        "ns_non_get && (ns_header_cookie||ns_header_pragma)".

                Default value: 0
                """
                return self.options['rule']

        def set_domain(self, domain):
                """
                The domain name. The string value can range to 63 characters.

                Default value: 0
                Minimum length =  1.
                """
                self.options['domain'] = domain

        def get_domain(self):
                """
                The domain name. The string value can range to 63 characters.

                Default value: 0
                Minimum length =  1.
                """
                return self.options['domain']

        def get_vstype(self):
                """
                Virtual server type.

                Default value: 0
                """
                return self.options['vstype']

        def get_hits(self):
                """
                Total number of hits.

                Default value: 0
                """
                return self.options['hits']

        def get_bindhits(self):
                """
                Total number of hits.

                Default value: 0
                """
                return self.options['bindhits']

        def get_labelname(self):
                """
                Name of the label invoked.

                Default value: 0
                """
                return self.options['labelname']

        def get_labeltype(self):
                """
                The invocation type.

                Default value: 0
                """
                return self.options['labeltype']

        def get_target(self):
                """
                Target flag.

                Default value: 0
                """
                return self.options['target']

        def get_priority(self):
                """
                Priority of bound policy.

                Default value: 0
                """
                return self.options['priority']

        # Operations methods
        @staticmethod
        def get(nitro, cspolicy):
                """
                Use this API to fetch cspolicy resource of given name.
                """
                __cspolicy = NSCSPolicy()
                __cspolicy.set_policyname(cspolicy.get_policyname())
                __cspolicy.get_resource(nitro)
                return __cspolicy

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured cspolicy resources.
                """
                __url = nitro.get_url() + NSCSPolicy.get_resourcetype()
                __json_cspolicies = nitro.get(__url).get_response_field(NSCSPolicy.get_resourcetype())
                __cspolicies = []
                for json_cspolicy in __json_cspolicies:
                        __cspolicies.append(NSCSPolicy(json_cspolicy))
                return __cspolicies

        @staticmethod
        def add(nitro, cspolicy):
                """
                Use this API to add cspolicy.
                """
                __cspolicy = NSCSPolicy()
                __cspolicy.set_policyname(cspolicy.get_policyname())
                __cspolicy.set_url(cspolicy.get_url())
                __cspolicy.set_rule(cspolicy.get_rule())
                __cspolicy.set_domain(cspolicy.get_domain())
                return __cspolicy.add_resource(nitro)

        @staticmethod
        def delete(nitro, cspolicy):
                """
                Use this API to delete cspolicy of a given name.
                """
                __cspolicy = NSCSPolicy()
                __cspolicy.set_policyname(cspolicy.get_policyname())
                nsresponse = __cspolicy.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, cspolicy):
                """
                Use this API to update a cspolicy of a given name.
                """
                __cspolicy = NSCSPolicy()
                __cspolicy.set_policyname(cspolicy.get_policyname())
                __cspolicy.set_url(cspolicy.get_url())
                __cspolicy.set_rule(cspolicy.get_rule())
                __cspolicy.set_domain(cspolicy.get_domain())
                return __cspolicy.update_resource(nitro)


        # No unset functionality for now.


from nsbaseresource import NSBaseResource
from nsconfig import NSConfig
from nscspolicy import NSCSPolicy
from nscsvserver import NSCSVServer
from nscsvservercspolicybinding import NSCSVServerCSPolicyBinding
from nscsvserverresponderpolicybinding import NSCSVServerResponderPolicyBinding
from nscsvserverrewritepolicybinding import NSCSVServerRewritePolicyBinding
from nslbvserver import NSLBVServer
from nslbvserverservicebinding import NSLBVServerServiceBinding
from nsresponderaction import NSResponderAction
from nsresponderpolicy import NSResponderPolicy
from nsresponderpolicylabel import NSResponderPolicyLabel
from nsresponderpolicylabelbinding import NSResponderPolicyLabelBinding
from nsresponderpolicycsvserverbinding import NSResponderPolicyCSVServerBinding
from nsrewritepolicy import NSRewritePolicy
from nsrewritepolicycsvserverbinding import NSRewritePolicyCSVServerBinding
from nsservice import NSService
from nsserver import NSServer
from nsservicegroup import NSServiceGroup

__all__ = [ 'NSBaseResource',
    'NSConfig',
    'NSCSPolicy',
    'NSCSVServer',
    'NSCSVServerCSPolicyBinding',
    'NSCSVServerResponderPolicyBinding',
    'NSCSVServerRewritePolicyBinding',
    'NSLBVServer',
    'NSLBVServerServiceBinding',
    'NSResponderAction',
    'NSResponderPolicy',
    'NSResponderPolicyLabel',
    'NSResponderPolicyLabelBinding',
    'NSResponderPolicyCSVServerBinding',
    'NSRewritePolicy',
    'NSRewritePolicyCSVServerBinding',
    'NSServer',
    'NSService',
    'NSServiceGroup'
 ]



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'

class NSRewritePolicy(NSBaseResource):

        # Configuration for content-switching policy resource.

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSRewritePolicy, self).__init__()
                self.options = {
                        'name' : '',
                        'rule' : '',
                        'action' : '',
                        'undefaction' : '',
                        'comment' : '',
                        'logaction' : '',
                        'newname' : '',
                        'hits' : '',
                        'undefhits' : '',
                        'description' : '',
                }

                self.resourcetype = NSRewritePolicy.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "rewritepolicy"

        def set_name(self, name):
                """
                Name of the rewrite policy.

                Default value: 0
                """
                self.options['name'] = name

        def get_name(self):
                """
                Name of the rewrite policy.

                Default value: 0
                """
                return self.options['name']

        def set_rule(self, rule):
                """
                Expression to be used by rewrite policy. It has to be a boolean PI rule expression.

                Default value: 0
                """
                self.options['rule'] = rule

        def get_rule(self):
                """
                Expression to be used by rewrite policy. It has to be a boolean PI rule expression.

                Default value: 0
                """
                return self.options['rule']

        def set_action(self, action):
                """
                Rewrite action to be used by the policy.

                Default value: 0
                """
                self.options['action'] = action

        def get_action(self):
                """
                Rewrite action to be used by the policy.

                Default value: 0
                """
                return self.options['action']

        def set_undefaction(self, undefaction):
                """
                A rewrite action, to be used by the policy when the rule evaluation turns out to be undefined.
                The undef action can be NOREWRITE, RESET or DROP.

                Default value: 0
                """
                self.options['undefaction'] = undefaction

        def get_undefaction(self):
                """
                A rewrite action, to be used by the policy when the rule evaluation turns out to be undefined.
                The undef action can be NOREWRITE, RESET or DROP.

                Default value: 0
                """
                return self.options['undefaction']

        def set_comment(self, comment):
                """
                Comments associated with this rewrite policy.

                Default value: 0
                """
                self.options['comment'] = comment

        def get_comment(self):
                """
                Comments associated with this rewrite policy.

                Default value: 0
                """
                return self.options['comment']

        def set_logaction(self, logaction):
                """
                The log action associated with the rewrite policy.

                Default value: 0
                """
                self.options['logaction'] = logaction

        def get_logaction(self):
                """
                The log action associated with the rewrite policy.

                Default value: 0
                """
                return self.options['logaction']

        def set_newname(self, newname):
                """
                The new name of the rewrite policy.

                Default value: 0
                Minimum length =  1.
                """
                self.options['newname'] = newname

        def get_newname(self):
                """
                The new name of the rewrite policy.

                Default value: 0
                Minimum length =  1.
                """
                return self.options['newname']

        def get_hits(self):
                """
                Number of hits.

                Default value: 0
                """
                return self.options['hits']

        def get_undefhits(self):
                """
                Number of undef hits.

                Default value: 0
                """
                return self.options['undefhits']

        def get_description(self):
                """
                Description of the policy.

                Default value: 0
                """
                return self.options['description']

        # Operations methods
        @staticmethod
        def get(nitro, rewritepolicy):
                """
                Use this API to fetch rewritepolicy resource of given name.
                """
                __rewritepolicy = NSRewritePolicy()
                __rewritepolicy.set_name(rewritepolicy.get_name())
                __rewritepolicy.get_resource(nitro)
                return __rewritepolicy

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured rewritepolicy resources.
                """
                __url = nitro.get_url() + NSRewritePolicy.get_resourcetype()
                __json_cspolicies = nitro.get(__url).get_response_field(NSRewritePolicy.get_resourcetype())
                __rewritepolicies = []
                for json_rewritepolicy in __json_cspolicies:
                        __rewritepolicies.append(NSRewritePolicy(json_rewritepolicy))
                return __rewritepolicies

        @staticmethod
        def add(nitro, rewritepolicy):
                """
                Use this API to add rewritepolicy.
                """
                __rewritepolicy = NSRewritePolicy()
                __rewritepolicy.set_name(rewritepolicy.get_name())
                __rewritepolicy.set_rule(rewritepolicy.get_rule())
                __rewritepolicy.set_action(rewritepolicy.get_action())
                __rewritepolicy.set_undefaction(rewritepolicy.get_undefaction())
                __rewritepolicy.set_comment(rewritepolicy.get_comment())
                __rewritepolicy.set_logaction(rewritepolicy.get_logaction())
                return __rewritepolicy.add_resource(nitro)

        @staticmethod
        def delete(nitro, rewritepolicy):
                """
                Use this API to delete rewritepolicy of a given name.
                """
                __rewritepolicy = NSRewritePolicy()
                __rewritepolicy.set_name(rewritepolicy.get_name())
                nsresponse = __rewritepolicy.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, rewritepolicy):
                """
                Use this API to update a rewritepolicy of a given name.
                """
                __rewritepolicy = NSRewritePolicy()
                __rewritepolicy.set_name(rewritepolicy.get_name())
                __rewritepolicy.set_rule(rewritepolicy.get_rule())
                __rewritepolicy.set_action(rewritepolicy.get_action())
                __rewritepolicy.set_undefaction(rewritepolicy.get_undefaction())
                __rewritepolicy.set_comment(rewritepolicy.get_comment())
                __rewritepolicy.set_logaction(rewritepolicy.get_logaction())
                return __rewritepolicy.update_resource(nitro)


        # No unset functionality for now.
  
        @staticmethod
        def rename(nitro, rewritepolicy):
                """
                Use this API to rename rewritepolicy.
                """
                __rewritepolicy = NSRewritePolicy()
                __rewritepolicy.set_name(rewritepolicy.get_name())
                __rewritepolicy.set_newname(rewritepolicy.get_newname())
                return __rewritepolicy.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'

class NSLBVServerServiceBinding(NSBaseResource):

        def __init__(self, json_data=None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSLBVServerServiceBinding, self).__init__()
                self.options = {
                        'servicename' : '',
                        'ipv46' : '',
                        'port' : '',
                        'servicetype' : '',
                        'curstate' : '',
                        'weight' : '',
                        'dynamicweight' : '',
                        'cookieipport' : '',
                        'vserverid' : '',
                        'name' : '',
                        'servicegroupname' : ''
                }


                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key]=json_data[key]

                self.resourcetype = NSLBVServerServiceBinding.get_resourcetype()

        @staticmethod
        def get_resourcetype():
                return "lbvserver_service_binding"


        # Getters and setters for configurable options
        def set_servicename(self, servicename):
                """
                The service name bound to the selected load balancing virtual server.
                Default value: 0
                Minimum length =  1.
                """
                self.options['servicename'] = servicename

        def get_servicename(self):
                """
                The service name bound to the selected load balancing virtual server.
                Default value: 0
                Minimum length =  1.
                """
                return self.options['servicename']

        def get_ipv46(self):
                """
                The IP address of the virtual server.
                Default value: 0
                """
                return self.options['ipv46']

        def get_port(self):
                """
                The IP address of the virtual server.
                Default value: 0
                """
                return self.options['port']

        def get_servicetype(self):
                """
                The service type.
                Default value: 0
                """
                return self.options['servicetype']

        def get_curstate(self):
                """
                Current LB vserver state.
                Default value: 0
                """
                return self.options['curstate']

        def set_weight(self, weight):
                """
                The weight for the specified service.
                Default value: 0
                Minimum value =  1
                Maximum value =  100
                """
                self.options['weight'] = weight

        def get_weight(self):
                """
                The weight for the specified service.
                Default value: 0
                Minimum value =  1
                Maximum value =  100
                """
                return self.options['weight']

        def get_dynamicweight(self):
                """
                Dynamic weight.
                Default value: 0
                """
                return self.options['dynamicweight']

        def get_cookieipport(self):
                """
                Encryped Ip address and port of the service that is inserted into the set-cookie http header.
                Default value: 0
                """
                return self.options['cookieipport']

        def get_vserverid(self):
                """
                Vserver Id.
                Default value: 0
                """
                return self.options['vserverid']

        def set_name(self, name):
                """
                The virtual server name to which the service is bound.
                Default value: 0
                Minimum length =  1.
                """
                self.options['name'] = name

        def get_name(self):
                """
                The virtual server name to which the service is bound.
                Default value: 0
                Minimum length =  1.
                """
                return self.options['name']

        def set_servicegroupname(self, servicegroupname):
                """
                The name of the service group that is bound.
                Default value: 0
                Minimum length =  1.
                """
                self.options['servicegroupname'] = servicegroupname

        def get_servicegroupname(self):
                """
                The name of the service group that is bound.
                Default value: 0
                Minimum length =  1.
                """
                return self.options['servicegroupname']


        # Operations methods
        @staticmethod
        def get(nitro, vserver_service_binding):
                """
                Use this API to fetch configured vserver_service_binding resources of a given name.
                """
                __url = nitro.get_url() + NSLBVServerServiceBinding.get_resourcetype() + "/" + vserver_service_binding.get_name()
                __json_services = nitro.get(__url).get_response_field(NSLBVServerServiceBinding.get_resourcetype())
                __services = []
                for json_service in __json_services:
                        __services.append(NSLBVServerServiceBinding(json_service))
                return __services


        @staticmethod
        def add(nitro, vserver_service_binding):
                """
                Use this API to add vserver_service_binding.
                """
                __vserver_service_binding = NSLBVServerServiceBinding()
                __vserver_service_binding.set_name(vserver_service_binding.get_name())
                __vserver_service_binding.set_servicename(vserver_service_binding.get_servicename())
                __vserver_service_binding.set_weight(vserver_service_binding.get_weight())
                __vserver_service_binding.set_servicegroupname(vserver_service_binding.get_servicegroupname())
                return __vserver_service_binding.update_resource(nitro)

        @staticmethod
        def delete(nitro, vserver_service_binding):
                """
                Use this API to delete vserver_service_binding of a given name.
                """
                __vserver_service_binding = NSLBVServerServiceBinding()
                __vserver_service_binding.set_name(vserver_service_binding.get_name())
                __vserver_service_binding.set_servicename(vserver_service_binding.get_servicename())
                __vserver_service_binding.set_servicegroupname(vserver_service_binding.get_servicegroupname())
                nsresponse = __vserver_service_binding.delete_resource(nitro)
                return nsresponse



from nsnitro import NSNitro
from nsutil import NSNitroError, NSNitroResponse
from nsresources import *
__all__ = ['NSNitro', 'NSNitroError', 'NSNitroResponse' ] + nsresources.__all__
__import__("pkg_resources").declare_namespace(__name__)



from nsbaseresource import NSBaseResource

__author__ = 'timl'

class NSResponderAction(NSBaseResource):

        # Configuration for responder actions.

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSResponderAction, self).__init__()
                self.options = {
                        'name' : '',
                        'newname' : '',
                        'type' : '',
                        'target' : '',
                        'bypasssafetycheck' : '',
                        'hits' : '',
                        'referencecount' : '',
                        'undefhits' : '',
                }

                self.resourcetype = NSResponderAction.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "responderaction"

        def set_name(self, name):
                """
                Name of the responder action.

                Default value: 0
                """
                self.options['name'] = name

        def get_name(self):
                """
                Name of the responder action.

                Default value: 0
                """
                return self.options['name']

        def set_type(self, action_type):
                """
                Type (respondwith, redirect) of responder action.

                Default value: 0
                """
                self.options['type'] = action_type

        def get_type(self):
                """
                Type (respondwith, redirect) of responder action.

                Default value: 0
                """
                return self.options['type']

        def set_target(self, target):
                """
                Target of responder action.

                Default value: 0
                """
                self.options['target'] = target

        def get_target(self):
                """
                Target of responder action.

                Default value: 0
                """
                return self.options['target']

        def set_newname(self, newname):
                """
                The new name of the responder action.

                Default value: 0
                Minimum length =  1.
                """
                self.options['newname'] = newname

        def get_newname(self):
                """
                The new name of the responder action.

                Default value: 0
                Minimum length =  1.
                """
                return self.options['newname']

        def get_hits(self):
                """
                Number of hits.

                Default value: 0
                """
                return self.options['hits']

        def get_undefhits(self):
                """
                Number of undef hits.

                Default value: 0
                """
                return self.options['undefhits']

        def get_referencecount(self):
                """
                Number of references to this action.

                Default value: 0
                """
                return self.options['referencecount']

        def set_bypasssafetycheck(self, bypasssafetycheck):
                """
                Bypass the safety check

                Default value: NO
                """
                valid = ('YES', 'NO')
                if bypasssafetycheck and bypasssafetycheck not in valid:
                     raise ValueError("bypasssafetycheck must be one of %s" %
                         ",".join(valid))
                self.options['bypasssafetycheck'] = bypasssafetycheck

        def get_bypasssafetycheck(self):
                """
                Bypass the safety check

                Default value: NO
                """
                return self.options['bypasssafetycheck']

        # Operations methods
        @staticmethod
        def get(nitro, responderaction):
                """
                Use this API to fetch responderaction resource of given name.
                """
                __responderaction = NSResponderAction()
                __responderaction.set_name(responderaction.get_name())
                __responderaction.get_resource(nitro)
                return __responderaction

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured responderaction resources.
                """
                __url = nitro.get_url() + NSResponderAction.get_resourcetype()
                __json_cspolicies = nitro.get(__url).get_response_field(NSResponderAction.get_resourcetype())
                __responderpolicies = []
                for json_responderaction in __json_cspolicies:
                        __responderpolicies.append(NSResponderAction(json_responderaction))
                return __responderpolicies

        @staticmethod
        def add(nitro, responderaction):
                """
                Use this API to add responderaction.
                """
                __responderaction = NSResponderAction()
                __responderaction.set_name(responderaction.get_name())
                __responderaction.set_type(responderaction.get_type())
                __responderaction.set_target(responderaction.get_target())
                __responderaction.set_bypasssafetycheck(responderaction.get_bypasssafetycheck())
                return __responderaction.add_resource(nitro)

        @staticmethod
        def delete(nitro, responderaction):
                """
                Use this API to delete responderaction of a given name.
                """
                __responderaction = NSResponderAction()
                __responderaction.set_name(responderaction.get_name())
                nsresponse = __responderaction.delete_resource(nitro)
                return nsresponse

        @staticmethod
        def update(nitro, responderaction):
                """
                Use this API to update a responderaction of a given name.
                """
                __responderaction = NSResponderAction()
                __responderaction.set_name(responderaction.get_name())
                __responderaction.set_target(responderaction.get_target())
                __responderaction.set_bypasssafetycheck(responderaction.get_bypasssafetycheck())
                return __responderaction.update_resource(nitro)


        # No unset functionality for now.
  
        @staticmethod
        def rename(nitro, responderaction):
                """
                Use this API to rename responderaction.
                """
                __responderaction = NSResponderAction()
                __responderaction.set_name(responderaction.get_name())
                __responderaction.set_newname(responderaction.get_newname())
                return __responderaction.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource

__author__ = 'timl'

class NSResponderPolicyLabel(NSBaseResource):

        # Configuration for responder policy labels

        def __init__(self, json_data = None):
                """
                Supplied with json_data the object can be pre-filled
                """
                super(NSResponderPolicyLabel, self).__init__()
                self.options = {
                        'labelname' : '',
                        'policylabeltype' : '',
                        'numpol' : '',
                        'hits' : '',
                        'priority' : '',
                }

                self.resourcetype = NSResponderPolicyLabel.get_resourcetype()

                if not (json_data is None):
                        for key in json_data.keys():
                                if self.options.has_key(key):
                                        self.options[key] = json_data[key]


        @staticmethod
        def get_resourcetype():
                return "responderpolicylabel"

        def set_labelname(self, labelname):
                """
                Name of the responder policy label.

                Default value: 0
                """
                self.options['labelname'] = labelname

        def get_labelname(self):
                """
                Name of the responder policy label.

                Default value: 0
                """
                return self.options['labelname']

        def set_policylabeltype(self, policylabeltype):
                """
                Policy label type  = "HTTP | OTHERTCP"

                Default value: 0
                """
                valid_types = ('HTTP', 'OTHERTCP')
                if policylabeltype and policylabeltype not in valid_types:
                    raise ValueError("policylabeltype must be one of %s" % \
                        ",".join(valid_types))
                self.options['policylabeltype'] = policylabeltype

        def get_policylabeltype(self):
                """
                Get the policylabeltype

                Default value: HTTP
                """
                return self.options['policylabeltype']

        def get_numpol(self):
                """
                Number of bound policies

                Default value: 0
                """
                return self.options['numpol']

        def get_hits(self):
                """
                Number of hits.

                Default value: 0
                """
                return self.options['hits']

        def set_priority(self, priority):
                """
                Set the priority for the responder policy label.

                Default value: 0
                """
                self.options['priority'] = priority

        def get_priority(self):
                """
                Priority of the responder policy label.

                Default value: 0
                """
                return self.options['priority']

        # Operations methods
        @staticmethod
        def get(nitro, responderpolicylabel):
                """
                Use this API to fetch responderpolicylabel resource of given name.
                """
                __responderpolicylabel = NSResponderPolicyLabel()
                __responderpolicylabel.set_labelname(responderpolicylabel.get_labelname())
                __responderpolicylabel.get_resource(nitro)
                return __responderpolicylabel

        @staticmethod
        def get_all(nitro):
                """
                Use this API to fetch all configured responderpolicylabel resources.
                """
                __url = nitro.get_url() + NSResponderPolicyLabel.get_resourcetype()
                __json_cspolicies = nitro.get(__url).get_response_field(NSResponderPolicyLabel.get_resourcetype())
                __responderpolicies = []
                for json_responderpolicylabel in __json_cspolicies:
                        __responderpolicies.append(NSResponderPolicyLabel(json_responderpolicylabel))
                return __responderpolicies

        @staticmethod
        def add(nitro, responderpolicylabel):
                """
                Use this API to add responderpolicylabel.
                """
                __responderpolicylabel = NSResponderPolicyLabel()
                __responderpolicylabel.set_labelname(responderpolicylabel.get_labelname())
                __responderpolicylabel.set_policylabeltype(responderpolicylabel.get_policylabeltype())
                __responderpolicylabel.set_priority(responderpolicylabel.get_priority())
                return __responderpolicylabel.add_resource(nitro)

        @staticmethod
        def delete(nitro, responderpolicylabel):
                """
                Use this API to delete responderpolicylabel of a given name.
                """
                __responderpolicylabel = NSResponderPolicyLabel()
                __name = responderpolicylabel.get_labelname()
                __responderpolicylabel.set_labelname(__name)
                nsresponse = __responderpolicylabel.delete_resource(nitro, __name)
                return nsresponse

        @staticmethod
        def update(nitro, responderpolicylabel):
                """
                Use this API to update a responderpolicylabel of a given name.
                """
                __responderpolicylabel = NSResponderPolicyLabel()
                __responderpolicylabel.set_labelname(responderpolicylabel.get_labelname())
                __responderpolicylabel.set_policylabeltype(responderpolicylabel.get_policylabeltype())
                __responderpolicylabel.set_priority(responderpolicylabel.get_priority())
                return __responderpolicylabel.update_resource(nitro)


        # No unset functionality for now.
  
        # No rename functionality for now.
